/// <reference types="node" />
import { EventEmitter2 } from 'eventemitter2';
import { EngineInterface } from '../types/engine';
import { Op, Path, Doc } from 'sharedb';
import { NodeInterface } from '../types/node';
import { DocInterface } from '../types/ot';
declare class Creator extends EventEmitter2 {
    private engine;
    private doc?;
    private dmp;
    private addedNodes;
    private cacheTransientElements?;
    timer: NodeJS.Timeout | null;
    lineStart: boolean;
    laterOps: Op[] | null;
    constructor(engine: EngineInterface, options: {
        doc?: DocInterface | Doc;
    });
    patchesToOps(path: Path, text1: string, text2: string): (import("sharedb/lib/sharedb").StringInsertOp | import("sharedb/lib/sharedb").StringDeleteOp)[];
    cacheAddedNode(node: Node): void;
    clearAddedNodeCache(): void;
    inAddedCache(node: Node): Node | undefined;
    isTransientMutation(record: MutationRecord, transientElements?: Array<Node>): boolean;
    /**
     * 从DOM变更记录中生产 ops （json格式的操作集合）
     * @param records DOM变更记录集合
     * @param path 路径
     * @param oldPath
     * @param node 开始遍历的节点，默认为编辑器根节点
     * @returns
     */
    makeOpsFromMutations(records: MutationRecord[], path?: Path, oldPath?: Path, node?: NodeInterface): import("sharedb/lib/sharedb").Op[];
    /**
     * 获取要移除节点的索引
     * @param record 当前记录
     * @param records 记录集合
     * @returns
     */
    getRemoveNodeIndex(record: MutationRecord, records: MutationRecord[]): number;
    getRemoveNodeIndexFromMutation(node: Node, target: Node, records: MutationRecord[]): number;
    /**
     * 处理DOM节点变更记录
     * @param records 记录集合
     */
    handleMutations(records: MutationRecord[]): void;
    normalizeOps(ops: Op[]): void;
    setDoc(doc: DocInterface | Doc): void;
    readyToEmitOps(ops: any[]): void;
}
export default Creator;
